/*******************************************************************************
*
*    File Name:   expaList.c
*
*
*******************************************************************************
*
*    Functions :  
*               metListInit()
*               metListIsEmpty()
*               metListMakeEmpty()
*               metListInsert()
*               metListRemove()
*               metListZeroth()
*               metListFirst()
*               metListFind()
*               metListAdvance()
*		rxnListInit()
*		rxnListInsert()
*               rxnListRemove()
*		rxnListFind()
*               rxnListAdvance()
*               rListInit()
*               rListIsEmpty()
*               rListMakeEmpty()
*               rListInsert()
*               rListRemove()
*               rListZeroth()
*               rListFirst()
*               rListFind()
*               rListAdvance()
*				
*
********************************************************************************
*
*   History:
*
*	Rev.	Date		Name	               Description
*	-------------------------------------------------------------------------
*	1.0	4/20/04         Steve Bell             Created
*******************************************************************************/
#define _EXPALIST_C

#include <stdlib.h>
#include <string.h>
#include "expa.h"
#include "expaList.h"



/* need to assume that header and current are initialized */
/*******************************************************************************
*      Function: void metListInit(struct metElt** hPtr, struct metElt** cPtr)
*
*   Description: Initializes a list 
*
*    Parameters: pointer to header, pointer to current element
*
*       Returns: nothing 
******************************************************************************/

void metListInit(struct metElt** hPtr, struct metElt** cPtr)
{
	struct metElt* header = *hPtr;
   	header->Next = NULL;
   	*cPtr = header;
}/* metListInit() */


/*******************************************************************************
*      Function: metListIsEmpty(struct metElt** hPtr)
*
*   Description: Function is used to query if the list 
*                is empty
*
*    Parameters: none
*
*       Returns: Function returns 'my_true' if list is empty, otherwise 'my_false' 
******************************************************************************/

BOOLEAN metListIsEmpty(struct metElt** hPtr)
{

   	return( (BOOLEAN) ( (*hPtr)->Next == NULL) );

} /* metListIsEmpty() */


/*******************************************************************************
*      Function: void metListMakeEmpty(struct metElt** hPtr, struct metElt** cPtr)
*
*   Description: Function empties the list 
*
*    Parameters: pointer to header, pointer to current
*
*       Returns: nothing
******************************************************************************/
void metListMakeEmpty(struct metElt** hPtr, struct metElt** cPtr)
{
	struct metElt *header = *hPtr;
	struct metElt *eltPtr;
	struct metElt *nextElt;

	for( eltPtr = header->Next; eltPtr != NULL; eltPtr = nextElt){
#ifdef DYNAMIC
		free(eltPtr->metName);
#endif
		nextElt = eltPtr->Next;
		free( (void *) eltPtr );
	}
	header->Next = NULL;
	*cPtr = header;
}/* metListMakeEmpty() */

/*********************************************************************************************************************
*      Function: BOOLEAN metListInsert(char* newMet, struct metElt** cPtr)
*
*   Description: Function inserts newMet at current
*                position of list and advances current
*
*    Parameters: row and row value of the element to be added to the list 
*
*       Returns: 'my_true' if memory is available, otherwise 'my_false'
***********************************************************************************************************************/
BOOLEAN metListInsert(char* newMet, struct metElt** cPtr)
{
	struct metElt* current = *cPtr;
	struct metElt* newElt;
	newElt = (struct metElt*) malloc( (unsigned) sizeof(struct metElt) );
	if( newElt == NULL ){
	       return my_false;
	}
	newElt->metName = newMet;
	newElt->Next = current->Next;
	current->Next = newElt;
	*cPtr = newElt;
	return my_true;
} /* metListInsert() */

/*************************************************************************************************
*      Function: BOOLEAN metListRemove(char* oldMet, struct metElt** hPtr, struct metElt** cPtr)
*
*   Description: Function removes oldMet from list and sets
*                current = header
*
*    Parameters: row of the column to be removed from the list;
*
*       Returns: 'my_true' if sucess, else 'my_false'
******************************************************************************/

BOOLEAN metListRemove(char* oldMet, struct metElt** hPtr, struct metElt** cPtr)
{
   struct metElt *eltPtr, *removedElt; 
   eltPtr = *hPtr;


   while( (eltPtr->Next != NULL) && ( strcmp( eltPtr->Next->metName, oldMet ) != 0 ) ){
        eltPtr = eltPtr->Next;
   }
   if( eltPtr->Next == NULL ){
       return my_false;
   }
   else{
       removedElt = eltPtr->Next;
       eltPtr->Next = eltPtr->Next->Next;
       free( (void *) removedElt);
       *cPtr = *hPtr;
       return my_true;
   }
} /* metListRemove() */

/*******************************************************************************
*      Function: void metListZeroth(struct metElt** hPtr, struct metElt** cPtr)
*
*   Description: Function sets current=header for the list 
*
*    Parameters: pointer to header, pointer to current element
*
*       Returns: nothing
******************************************************************************/
void metListZeroth(struct metElt** hPtr, struct metElt** cPtr)
{
	*cPtr = *hPtr;

}/* metListZeroth() */

/*******************************************************************************
*      Function: BOOLEAN metListFirst(struct metElt** hPtr, struct metElt** cPtr)
*
*   Description: Function sets current to point to first element in
*                the list 
*
*    Parameters: pointer to header, pointer to current element
*
*       Returns: 'my_true' if list in nonempty, else 'my_false'
******************************************************************************/
BOOLEAN metListFirst(struct metElt** hPtr, struct metElt** cPtr)
{
	struct metElt* header = *hPtr;
   	if( header->Next == NULL ){
       		return my_false;
   	}
   	else{
       		*cPtr = header->Next;
       		return my_true;
   	}
   
} /* metListFirst() */

/********************************************************************************************
*      Function: BOOLEAN metListFind(char* findMet, struct metElt** hPtr, struct metElt** cPtr)
*
*   Description: Function finds findMet in list and sets current to point to it
*
*    Parameters: the element to be found 
*
*       Returns: 'my_true' if found, else 'my_false'
**********************************************************************************************/
BOOLEAN metListFind(char* findMet, struct metElt** hPtr, struct metElt** cPtr)
{
    struct metElt *header = *hPtr;
    struct metElt *eltPtr;
    eltPtr = header->Next;

    while( (eltPtr != NULL) && ( strcmp( eltPtr->metName, findMet ) != 0 ) ){
         eltPtr = eltPtr->Next;
    }
    if( eltPtr == NULL ){
         return my_false;
    }

    *cPtr = eltPtr;
    return my_true;
}/* metListFind() */


/*******************************************************************************
*      Function: BOOLEAN metListAdvance(struct metElt** cPtr)
*
*   Description: Function advances current one step in list 
*
*       Returns: 'my_true' if success, else 'NULL'
******************************************************************************/
BOOLEAN metListAdvance(struct metElt** cPtr)
{
	struct metElt* current = *cPtr;
    	if( current->Next ==  NULL ){
        	return my_false;
    	}
    	else{
        	*cPtr = current->Next;
                return my_true;
    	}
}/* metListAdvance() */

/*******************************************************************************************
*      Function: BOOLEAN rxnListInit(struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr)
*
*   Description: Initializes a rxn list 
*
*    Parameters: pointer to header, pointer to current element
*
*       Returns: my_false if unable to allocate memory
*********************************************************************************************/

BOOLEAN rxnListInit(struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr)
{
	struct rxnElt* header = *rxnHPtr;
   	header->Next = NULL;
	header->headerPtr = (struct rElt**) malloc( (unsigned) sizeof(struct rElt*) );
	if( header->headerPtr == NULL ){
		return my_false;
	}
	header->currentPtr = (struct rElt**) malloc( (unsigned) sizeof(struct rElt*) );
	if( header->currentPtr == NULL ){
		return my_false;
	}
	*(header->headerPtr) = (struct rElt*) malloc( (unsigned) sizeof(struct rElt) );
	if( *(header->headerPtr) == NULL ){
		return my_false;
	}
	*(header->currentPtr) = (struct rElt*) malloc( (unsigned) sizeof(struct rElt) );
	if( *(header->currentPtr) == NULL ){
		return my_false;
	}
        rListInit( header->headerPtr, header->currentPtr );
   	*rxnCPtr = header;
	return my_true;
}/* rxnListInit() */


/*********************************************************************************************************************
*      Function: BOOLEAN rxnListInsert(char* newRxn, struct rxnElt** rxnCPtr)
*
*   Description: Function inserts newRxn at current position of list and advances current
*
*    Parameters: name of the rxn element to be added to the list 
*
*       Returns: 'my_true' if memory is available, otherwise 'my_false'
*
*       Note: The list corresponding to headerPtr and currentPtr in rxnElt needs to be inserted as well, i.e.,
*       the function rListInsert() should be called after this function
***********************************************************************************************************************/
BOOLEAN rxnListInsert(char* newRxn, struct rxnElt** rxnCPtr)
{
	struct rxnElt* current = *rxnCPtr;
	struct rxnElt *newElt;
	newElt = (struct rxnElt*) malloc( sizeof(struct rxnElt) );
	if( newElt == NULL ){
	       return my_false;
	}
	newElt->rxnName = newRxn;
	newElt->Next = current->Next;
	newElt->headerPtr = (struct rElt**) malloc( (unsigned) sizeof(struct rElt*) );
	if( newElt->headerPtr == NULL ){
		return my_false;
	}
	newElt->currentPtr = (struct rElt**) malloc( (unsigned) sizeof(struct rElt*) );
	if( newElt->currentPtr == NULL ){
		return my_false;
	}
	*(newElt->headerPtr) = (struct rElt*) malloc( (unsigned) sizeof(struct rElt) );
	if( *(newElt->headerPtr) == NULL ){
		return my_false;
	}
	*(newElt->currentPtr) = (struct rElt*) malloc( (unsigned) sizeof(struct rElt) );
	if( *(newElt->currentPtr) == NULL ){
		return my_false;
	}
	rListInit(newElt->headerPtr, newElt->currentPtr);

	current->Next = newElt;
	*rxnCPtr = newElt;
	return my_true;
} /* rxnListInsert() */

/***********************************************************************************************************
*      Function: BOOLEAN rxnListFind(char* findRxn, struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr)
*
*   Description: Function finds findRxn in list and sets current to point to it
*
*    Parameters: the element to be found 
*
*       Returns: 'my_true' if found, else 'my_false'
************************************************************************************************************/
BOOLEAN rxnListFind(char* findRxn, struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr)
{
    struct rxnElt *header = *rxnHPtr;
    struct rxnElt *eltPtr;
    eltPtr = header->Next;

    while( (eltPtr != NULL) && ( strcmp( eltPtr->rxnName, findRxn ) != 0 ) ){
         eltPtr = eltPtr->Next;
    }
    if( eltPtr == NULL ){
         return my_false;
    }

    *rxnCPtr = eltPtr;
    return my_true;
}/* rxnListFind() */

/*************************************************************************************************************
*      Function: BOOLEAN rxnListRemove(char* oldRxn, struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr)
*
*   Description: Function removes oldRxn from list and sets
*                current = header
*
*    Parameters: rxn to be removed from the list;
*
*       Returns: 'my_true' if sucess, else 'my_false'
******************************************************************************/

BOOLEAN rxnListRemove(char* oldRxn, struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr)
{
   struct rxnElt *eltPtr, *removedElt; 
   eltPtr = *rxnHPtr;

   while( (eltPtr->Next != NULL) && ( strcmp( eltPtr->Next->rxnName, oldRxn ) != 0 ) ){
        eltPtr = eltPtr->Next;
   }
   if( eltPtr->Next == NULL ){
       return my_false;
   }
   else{
       removedElt = eltPtr->Next;
       eltPtr->Next = eltPtr->Next->Next;
       rListMakeEmpty( removedElt->headerPtr, removedElt->currentPtr );	   
       free( (void *) removedElt);
       *rxnCPtr = *rxnHPtr;
       return my_true;
   }
} /* ListRemove() */

/*******************************************************************************
*      Function: void rxnListMakeEmpty(struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr)
*
*   Description: Function empties the list 
*
*    Parameters: pointer to header, pointer to current
*
*       Returns: nothing
******************************************************************************/
void rxnListMakeEmpty(struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr)
{
	struct rxnElt *header = *rxnHPtr;
	struct rxnElt *eltPtr;
	struct rxnElt *nextElt;

	for( eltPtr = header->Next; eltPtr != NULL; eltPtr = nextElt){
		rListMakeEmpty(eltPtr->headerPtr, eltPtr->currentPtr);
#ifdef DYNAMIC
		free(eltPtr->rxnName);
#endif
		nextElt = eltPtr->Next;
		free( (void *) eltPtr );
	}
	header->Next = NULL;
	*rxnCPtr = header;
}/* rListMakeEmpty() */

/*******************************************************************************
*      Function: BOOLEAN rxnListAdvance(struct rxnElt** rxnCPtr)
*
*   Description: Function advances current one step in list 
*
*       Returns: 'my_true' if success, else 'NULL'
******************************************************************************/
BOOLEAN rxnListAdvance(struct rxnElt** rxnCPtr)
{
	struct rxnElt* current = *rxnCPtr;
    	if( current->Next ==  NULL ){
        	return my_false;
    	}
    	else{
        	*rxnCPtr = current->Next;
                return my_true;
    	}
}/* rxnListAdvance() */

/*******************************************************************************
*      Function: void rxnListZeroth(struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr)
*
*   Description: Function sets current=header for the list 
*
*    Parameters: pointer to header, pointer to current element
*
*       Returns: nothing
******************************************************************************/
void rxnListZeroth(struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr)
{
	*rxnCPtr = *rxnHPtr;

}/* rxnListZeroth() */

/* need to assume that header and current are initialized */
/*******************************************************************************
*      Function: void rListInit(struct rElt** hPtr, struct rElt** cPtr)
*
*   Description: Initializes a list 
*
*    Parameters: pointer to header, pointer to current element
*
*       Returns: nothing 
******************************************************************************/

void rListInit(struct rElt** hPtr, struct rElt** cPtr)
{
	struct rElt* header = *hPtr;
   	header->Next = NULL;
   	*cPtr = header;
}/* rListInit() */


/*******************************************************************************
*      Function: BOOLEAN rListIsEmpty(struct rElt** hPtr)
*
*   Description: Function is used to query if the list 
*                is empty
*
*    Parameters: none
*
*       Returns: Function returns 'my_true' if list is empty, otherwise 'my_false' 
******************************************************************************/

BOOLEAN rListIsEmpty(struct rElt** hPtr)
{

   	return( (BOOLEAN) ( (*hPtr)->Next == NULL) );

} /* rListIsEmpty() */


/*******************************************************************************
*      Function: void rListMakeEmpty(struct rElt** hPtr, struct rElt** cPtr)
*
*   Description: Function empties the list 
*
*    Parameters: pointer to header, pointer to current
*
*       Returns: nothing
******************************************************************************/
void rListMakeEmpty(struct rElt** hPtr, struct rElt** cPtr)
{
	struct rElt *header = *hPtr;
	struct rElt *eltPtr;
	struct rElt *nextElt;

	for( eltPtr = header->Next; eltPtr != NULL; eltPtr = nextElt){

		/* we don't need to do: free(eltPtr->metName) since this is done in metListMakeEmpty() */
		nextElt = eltPtr->Next;
		free( (void *) eltPtr );
	}
	header->Next = NULL;
	*cPtr = header;
}/* rListMakeEmpty() */

/*********************************************************************************************************************
*      Function: BOOLEAN rListInsert(char* newMet, XTYPE metCoeff, struct rowElt** cPtr)
*
*   Description: Function inserts newMet and metCoeff at current
*                position of list and advances current
*
*    Parameters: met name and coefficient value of the element to be added to the list 
*
*       Returns: 'my_true' if memory is available, otherwise 'my_false'
***********************************************************************************************************************/
BOOLEAN rListInsert(char* newMet, XTYPE metCoeff,  struct rElt** cPtr)
{
	struct rElt* current = *cPtr;
	struct rElt *newElt;
	newElt = (struct rElt*) malloc( (unsigned) sizeof(struct rElt) );
	if( newElt == NULL ){
	       return my_false;
	}
	newElt->metName = newMet;
	newElt->coeff = metCoeff;
	newElt->Next = current->Next;
	current->Next = newElt;
	*cPtr = newElt;
	return my_true;
} /* rListInsert() */

/*************************************************************************************************
*      Function: BOOLEAN rListRemove(char* oldMet, struct rElt** hPtr, struct rElt** cPtr)
*
*   Description: Function removes oldMet from list and sets
*                current = header
*
*    Parameters: row of the column to be removed from the list;
*
*       Returns: 'my_true' if sucess, else 'my_false'
******************************************************************************/

BOOLEAN rListRemove(char* oldMet, struct rElt** hPtr, struct rElt** cPtr)
{
   struct rElt *eltPtr, *removedElt; 
   eltPtr = *hPtr;


   while( (eltPtr->Next != NULL) && ( strcmp( eltPtr->Next->metName, oldMet ) != 0 ) ){
        eltPtr = eltPtr->Next;
   }
   if( eltPtr->Next == NULL ){
       return my_false;
   }
   else{
       removedElt = eltPtr->Next;
       eltPtr->Next = eltPtr->Next->Next;
       free( (void *) removedElt);
       *cPtr = *hPtr;
       return my_true;
   }
} /* rListRemove() */

/*******************************************************************************
*      Function: void rListZeroth(struct rElt** hPtr, struct rElt** cPtr)
*
*   Description: Function sets current=header for the list 
*
*    Parameters: pointer to header, pointer to current element
*
*       Returns: nothing
******************************************************************************/
void rListZeroth(struct rElt** hPtr, struct rElt** cPtr)
{
	*cPtr = *hPtr;

}/* rListZeroth() */

/*******************************************************************************
*      Function: BOOLEAN rListFirst(struct rElt** hPtr, struct rElt** cPtr)
*
*   Description: Function sets current to point to first element in
*                the list 
*
*    Parameters: pointer to header, pointer to current element
*
*       Returns: 'my_true' if list in nonempty, else 'my_false'
******************************************************************************/
BOOLEAN rListFirst(struct rElt** hPtr, struct rElt** cPtr)
{
	struct rElt* header = *hPtr;
   	if( header->Next == NULL ){
       		return my_false;
   	}
   	else{
       		*cPtr = header->Next;
       		return my_true;
   	}
   
} /* rListFirst() */

/********************************************************************************************
*      Function: BOOLEAN rListFind(char* findMet, struct RowElt** hPtr, struct RowElt** cPtr)
*
*   Description: Function finds findMet in list and sets current to point to it
*
*    Parameters: the element to be found 
*
*       Returns: 'my_true' if found, else 'my_false'
**********************************************************************************************/
BOOLEAN rListFind(char* findMet, struct rElt** hPtr, struct rElt** cPtr)
{
    struct rElt *header = *hPtr;
    struct rElt *eltPtr;
    eltPtr = header->Next;

    while( (eltPtr != NULL) && ( strcmp( eltPtr->metName, findMet ) != 0 ) ){
         eltPtr = eltPtr->Next;
    }
    if( eltPtr == NULL ){
         return my_false;
    }

    *cPtr = eltPtr;
    return my_true;
}/* rListFind() */


/*******************************************************************************
*      Function: BOOLEAN RowListAdvance(struct RowElt** cPtr)
*
*   Description: Function advances current one step in list 
*
*       Returns: 'my_true' if success, else 'NULL'
******************************************************************************/
BOOLEAN rListAdvance(struct rElt** cPtr)
{
	struct rElt* current = *cPtr;
    	if( current->Next ==  NULL ){
        	return my_false;
    	}
    	else{
        	*cPtr = current->Next;
                return my_true;
    	}
}/* RowListAdvance() */
